ResetTimesDialog = class(Turbine.UI.Lotro.Window);

function ResetTimesDialog:Constructor(event)
    Turbine.UI.Lotro.Window.Constructor(self);
    local prevContext = L:SetContext("/ResetTimesDialog");
    self:SetText(L:GetText("Title"));
    self:SetVisible(true);
    self.event, self.window = event, event.tab.window;

    -- If the event is deleted, we should close this window
    self.eventDestroyedFunc = function()
        self:Close();
    end
    -- If the window is closed, we should unregister the callback
    self.Closing = function()
        RemoveCallback(event, "Destroyed", self.eventDestroyedFunc);
        self:SetWantsUpdates(false);
    end
    -- Register the callback now, but first make sure it isn't already registered
    self.Closing();
    AddCallback(event, "Destroyed", self.eventDestroyedFunc);

    -- Make this window prevent access to the main window while it's open
    self:SetZOrder(self.window:GetZOrder() + 1);
    self.window:WaitForChild(self);

    local timeSpec = self.event:GetResetTimes();
    self.resetTimesCtl = ResetTimesControl(timeSpec, true);
    AddCallback(self.resetTimesCtl, "SizeChanged", function()
        self:DoLayout();
    end);
    AddCallback(self.resetTimesCtl, "ValidChanged", function (_, valid)
        self.okButton:SetEnabled(valid);
    end);

    self.window = win;
    self.id = id;
    self.settings = settings;

    -- Handle ENTER and ESC keys
    EnableEnterEscHandling(self);
    L:SetContext(prevContext);
    self:DoLayout();
    
    -- Position this window so its center is at the mouse position
    local width, height = self:GetSize();
    local x, y = Turbine.UI.Display.GetMousePosition();
    local left = math.max(0, x - math.floor(width / 2));
    local top = math.max(0, y - math.floor(height / 2));
    self:SetPosition(left, top);
    
    -- Handle ENTER and ESC keys
    EnableEnterEscHandling(self);
end

function ResetTimesDialog:DoLayout()
    local xmargin = 28;
    local top = 39;

    self:GetControls():Clear();
    collectgarbage();

    self.resetTimesCtl:SetParent(self);
    self.resetTimesCtl:SetPosition(xmargin, top);
    width = self.resetTimesCtl:GetWidth() + 2 * xmargin;
    top = top + self.resetTimesCtl:GetHeight() + 10; -- skip some space

    local buttons = Turbine.UI.Control();
    buttons:SetParent(self);
    buttons:SetTop(top);

    local function AddButton(parent, left, top, width, text)
        local button = Turbine.UI.Lotro.Button();
        button:SetParent(parent);
        button:SetText(text);
        button:SetPosition(left, top);
        button:SetWidth(width);
        return button;
    end

    self.okButton = AddButton(buttons, 0, 0, 50, L:GetText("/ExpireTimeDialog/Ok"));
    self.okButton.SetEnabled = function(_, enable)
        -- Lotro.Button can't get mouse events while disabled.  Need to superimpose another control.
        if (enable and not self.okButton:IsEnabled()) then
            self.okMask:SetParent(nil);
            self.okMask = nil;
        elseif (not enable and self.okButton:IsEnabled()) then
            self.okMask = Turbine.UI.Control();
            self.okMask:SetParent(self.okButton);
            self.okMask:SetSize(self.okButton:GetSize());
            Thurallor.UI.Tooltip(L:GetText("/ResetTimesDialog/NotOkTooltip")):Attach(self.okMask);
        end
        Turbine.UI.Lotro.Button.SetEnabled(self.okButton, enable);
    end
    self.okButton.Click = function()
        local timeSpec = self.resetTimesCtl:GetTimeSpec();
        DoCallbacks(self, "Ok", { resetTimes = timeSpec });
        self:Close();
    end
    left = self.okButton:GetWidth() + 4;
    
    self.cancelButton = AddButton(buttons, left, 0, L:GetText("/ExpireTimeDialog/CancelWidth"), L:GetText("/ExpireTimeDialog/Cancel"));
    self.cancelButton.Click = function()
        self:Close();
    end
    buttons:SetSize(left + self.cancelButton:GetWidth(), self.cancelButton:GetHeight());
    buttons:SetLeft(math.floor(0.5 + (width - buttons:GetWidth()) / 2));
    top = top + buttons:GetHeight();

    local height = top + 22;
    self:SetSize(width, height);
    local center = math.floor(0.5 + width / 2)
end

function ResetTimesDialog:EnterKeyPressed()
    DoCallbacks(self.okButton, "Click");
end

function ResetTimesDialog:EscapeKeyPressed()
    self:Close();
end
